"""
sites.py

Created by Daniel Magee on 2008-01-29.
Copyright (c) 2008 UCO/Lick Observatory. All rights reserved.
"""

import ephem as E
import math

class Site(object):
    """Observatory site class."""
    def __init__(self, siteid):
        try:
            self.siteid = siteid
        except KeyError:
            print 'ERROR: site %s not found.' % siteid
    
    def __setattr__(self, name, value):
        if name == 'siteid':
            object.__setattr__(self, name, value)
            if value is not None:
                for k,v in self._siteData[value].items():
                    object.__setattr__(self, k, v)
        else:
            object.__setattr__(self, name, value)

    def observer(self):
        """Returns a pyephem Observer object for a site."""
        obs = E.Observer()
        obs.long = -1*E.degrees(self._siteData[self.siteid]['longitude'])
        obs.lat = E.degrees(self._siteData[self.siteid]['latitude'])
        obs.elevation = self._siteData[self.siteid]['elevation']
        obs.temp = self._siteData[self.siteid]['temperature']
        obs.pressure = self._siteData[self.siteid]['pressure']
        obs.epoch = 2000.0
        obs.horizon = -1*math.sqrt(2*obs.elevation/E.earth_radius)
        return obs

    def sites(self):
        """Returns a list of sites."""
        sitekeys = self._siteData.keys()
        sitekeys.sort()
        site_names = []
        for s in sitekeys:
            site_names.append([s, self._siteData[s]['name']])
        return site_names
        
    
    _siteData = {
		#logitudes +W -E, latitudes +N -S
        'kpno' : {'name':'Kitt Peak National Observatory',
			  'longitude':'111:36.0',
			  'latitude':'31:57.8',
			  'elevation':2120.,
			  'temperature':15.0,
			  'pressure':760.0,
              'timezone':'US/Mountain',
              'tzname':'Mountain'},
		'ctio' : {'name':'Cerro Tololo Interamerican Observatory',
			  'longitude':'70:48:54.00',
			  'latitude':'-30:09:55.00',
			  'elevation':2215.,
			  'temperature':7.0,
			  'pressure':740.0,
              'timezone':'Chile/Continental',
              'tzname':'Chilean'},
		'lasilla' : {'name':'European Southern Observatory: La Silla',
			  'longitude':'70:43.8',
			  'latitude':'-29:15.4',
			  'elevation':2347,
			  'temperature':10.0,
			  'pressure':770.0,
              'timezone':'Chile/Continental',
              'tzname':'Chilean'},
		'paranal' : {'name':'European Southern Observatory: Paranal',
			  'longitude':'70:24.2',
			  'latitude':'-24:37.5',
			  'elevation':2635,
			  'temperature':10.0,
			  'pressure':750.0,
                          'timezone':'Chile/Continental',
                          'tzname':'Chilean'},
		'lick' : {'name':'Lick Observatory',
			  'longitude':'121:38.2',
			  'latitude':'37:20.6',
			  'elevation':1290,
			  'temperature':15.0,
			  'pressure':1000.0,
                          'timezone':'US/Pacific',
                          'tzname':'Pacific'},
		'mmto' : {'name':'MMT Observatory',
			  'longitude':'110:53.1',
			  'latitude':'31:41.3',
			  'elevation':2600,
			  'temperature':14.0,
			  'pressure':980.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'},
		'cfht' : {'name':'Canada-France-Hawaii Telescope',
			  'longitude':'155:28.3',
			  'latitude':'19:49.6',
			  'elevation':4215,
			  'temperature':0.0,
			  'pressure':615.0,
                          'timezone':'US/Aleutian',
                          'tzname':'Hawaiian'},
		'lapalma' : {'name':'Roque de los Muchachos, La Palma',
			  'longitude':'17:52.8',
			  'latitude':'28:45.5',
			  'elevation':2327,
			  'temperature':8.0,
			  'pressure':768.0,
                          'timezone':'Atlantic/Canary',
                          'tzname':'Canary Islands'},
		'mso' : {'name':'Mt. Stromlo Observatory',
			  'longitude':'210:58:32.4',
			  'latitude':'-35:19:14.34',
			  'elevation':767,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'Australia/Sydney',
                          'tzname':'Australian'},
		'sso' : {'name':'Siding Spring Observatory',
			  'longitude':'210:56:19.70',
			  'latitude':'-31:16:24.10',
			  'elevation':1149,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'Australia/Sydney',
                          'tzname':'Australian'},
		'aao' : {'name':'Anglo-Australian Observatory',
			  'longitude':'210:56:2.09',
			  'latitude':'-31:16:37.34',
			  'elevation':1164,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'Australia/Sydney',
                          'tzname':'Australian'},
		'mcdonald' : {'name':'McDonald Observatory',
			  'longitude':'104:01:18.00',
			  'latitude':'30:40:18.00',
			  'elevation':2075,
			  'temperature':10.0,
			  'pressure':800.0,
                          'timezone':'US/Central',
                          'tzname':'Central'},
		'lco' : {'name':'Las Campanas Observatory',
			  'longitude':'70:42.1',
			  'latitude':'-29:0.2',
			  'elevation':2282,
			  'temperature':7.0,
			  'pressure':760.0,
                          'timezone':'Chile/Continental',
                          'tzname':'Chilean'},
		'mtbigelow' : {'name':'Catalina Observatory: 61 inch telescope',
			  'longitude':'110:43.9',
			  'latitude':'32:25.0',
			  'elevation':2510.,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'},
		'dao' : {'name':'Dominion Astrophysical Observatory',
			  'longitude':'123:25.0',
			  'latitude':'48:31.3',
			  'elevation':229,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Pacific',
                          'tzname':'Pacific'},
		'spm' : {'name':'Observatorio Astronomico Nacional, San Pedro Martir',
			  'longitude':'115:29:13',
			  'latitude':'31:01:45',
			  'elevation':2830,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Pacific',
                          'tzname':'Pacific'},
		'tona' : {'name':'Observatorio Astronomico Nacional, Tonantzintla',
			  'longitude':'98:18:50',
			  'latitude':'19:01:58',
			  'elevation':0,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Central',
                          'tzname':'Central'},
		'palomar' : {'name':'The Hale Telescope',
			  'longitude':'116:51:46.80',
			  'latitude':'33:21:21.6',
			  'elevation':1706.,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Pacific',
                          'tzname':'Pacific'},
		'mdm' : {'name':'Michigan-Dartmouth-MIT Observatory',
			  'longitude':'111:37.0',
			  'latitude':'31:57.0',
			  'elevation':1938.5,
			  'temperature':15.0,
			  'pressure':760.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'},
		'bmo' : {'name':'Black Moshannon Observatory',
			  'longitude':'78:00.3',
			  'latitude':'40:55.3',
			  'elevation':738,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Eastern',
                          'tzname':'Eastern'},
		'keck' : {'name':'W. M. Keck Observatory',
			  'longitude':'155:28:28.11',
			  'latitude':'19:49:34.51',
			  'elevation':4160,
			  'temperature':0.0,
			  'pressure':615.0,
                          'timezone':'US/Aleutian',
                          'tzname':'Hawaiian'},
		'subaru' : {'name':'Subaru Telescope',
			  'longitude':'155:28:33.67',
			  'latitude':'19:49:31.81',
			  'elevation':4163,
			  'temperature':0.0,
			  'pressure':615.0,
                          'timezone':'US/Aleutian',
                          'tzname':'Hawaiian'},
		'apo' : {'name':'Apache Point Observatory',
			  'longitude':'105:49.2',
			  'latitude':'32:46.8',
			  'elevation':2798.,
			  'temperature':10.0,
			  'pressure':760.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'},
		'lowell' : {'name':'Lowell Observatory',
			  'longitude':'111:32.1',
			  'latitude':'35:05.8',
			  'elevation':2198.,
			  'temperature':15.0,
			  'pressure':1010.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'},
		'flwo' : {'name':'Whipple Observatory',
			  'longitude':'110:52:39',
			  'latitude':'31:40:51.4',
			  'elevation':2320,
			  'temperature':14.0,
			  'pressure':980.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'},
		'irtf' : {'name':'NASA Infrared Telescope Facility',
			  'longitude':'155:28:19.19564',
			  'latitude':'19:49:34.38594',
			  'elevation':4168,
			  'temperature':0.0,
			  'pressure':615.0,
                          'timezone':'US/Aleutian',
                          'tzname':'Hawaiian'},
		'gemini-north' : {'name':'Gemini North Observatory',
			  'longitude':'155:28:08.56831',
			  'latitude':'19:49:25.68521',
			  'elevation':4213.4,
			  'temperature':0.0,
			  'pressure':615.0,
                          'timezone':'US/Aleutian',
                          'tzname':'Hawaiian'},
		'gemini-south' : {'name':'Gemini South Observatory',
			  'longitude':'70:43.4',
			  'latitude':'-30:13.7',
			  'elevation':2737.,
			  'temperature':5.0,
			  'pressure':735.0,
                          'timezone':'Chile/Continental',
                          'tzname':'Chilean'},
		'esontt' : {'name':'European Southern Observatory, NTT, La Silla',
			  'longitude':'70:43:54.272',
			  'latitude':'-29:15:18.440',
			  'elevation':2375,
			  'temperature':10.0,
			  'pressure':770.0,
                          'timezone':'Chile/Continental',
                          'tzname':'Chilean'},
		'eso36m' : {'name':'European Southern Observatory, 3.6m Telescope, La Silla',
			  'longitude':'70:43:46.606',
			  'latitude':'-29:15:25.814',
			  'elevation':2400,
			  'temperature':10.0,
			  'pressure':770.0,
                          'timezone':'Chile/Continental',
                          'tzname':'Chilean'},
		'vlt' : {'name':'European Southern Observatory, VLT, Paranal',
			  'longitude':'70:24:10.1',
			  'latitude':'-24:37:31.5',
			  'elevation':2648,
			  'temperature':10.0,
			  'pressure':750.0,
                          'timezone':'Chile/Continental',
                          'tzname':'Chilean'},
		'mgo' : {'name':'Mount Graham Observatory',
			  'longitude':'109:53:30',
			  'latitude':'32:42:06',
			  'elevation':3181,
			  'temperature':15.0,
			  'pressure':760.0,
                          'timezone':'US/Mountain',
                          'tzname':'Mountain'}
            }